package cz.muni.fi.pb162.test3;

import junit.framework.TestCase;

public class RailwayStationTest extends TestCase {
    
    public RailwayStationTest(String testName) {
        super(testName);
    }

    Train osobak = new Train("osobak");
    Train orientExpress = new Train("Orient express");
    Train brnenskyDrak = new Train("IC 570 Brnensky drak");
    Train vysehrad = new Train("EC 134 Vysehrad");
    

    public void testConstructor() {
        try {
            new RailwayStationImpl(0);
            fail("Je mozne vytvorit stanici s nulovym poctem koleji");
        } catch (IllegalArgumentException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }
        try {
            new RailwayStationImpl(-1);
            fail("Je mozne vytvorit stanici se zapornym poctem koleji");
        } catch (IllegalArgumentException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }
    }
    
    public void testGetTracksCount() {
        RailwayStation station = new RailwayStationImpl(17);
        assertEquals(17,station.getTracksCount());
        assertEquals(17,station.getFreeTracksCount());
        station.enter(orientExpress);
        assertEquals(17,station.getTracksCount());
        assertEquals(16,station.getFreeTracksCount());
        station.enter(vysehrad);
        assertEquals(17,station.getTracksCount());
        assertEquals(15,station.getFreeTracksCount());
        station.leave(vysehrad);
        assertEquals(17,station.getTracksCount());
        assertEquals(16,station.getFreeTracksCount());
        station.enter(brnenskyDrak);
        assertEquals(17,station.getTracksCount());
        assertEquals(15,station.getFreeTracksCount());
        station.leave(orientExpress);
        assertEquals(17,station.getTracksCount());
        assertEquals(16,station.getFreeTracksCount());
        station.leave(brnenskyDrak);
        assertEquals(17,station.getTracksCount());
        assertEquals(17,station.getFreeTracksCount());
    } 

    public void testGetTrackNumber() {
        RailwayStation station = new RailwayStationImpl(17);
        assertEquals(0,station.getTrackNumber(orientExpress));
        assertEquals(0,station.getTrackNumber(vysehrad));
        station.enter(orientExpress);
        assertEquals(1,station.getTrackNumber(orientExpress));
        station.enter(vysehrad);
        assertEquals(1,station.getTrackNumber(orientExpress));
        assertEquals(2,station.getTrackNumber(vysehrad));
        station.leave(orientExpress);
        assertEquals(0,station.getTrackNumber(orientExpress));
        assertEquals(2,station.getTrackNumber(vysehrad));
        station.leave(vysehrad);
        assertEquals(0,station.getTrackNumber(orientExpress));
        assertEquals(0,station.getTrackNumber(vysehrad));
        
        try {
            station.getTrackNumber(null);
            fail();
        } catch (NullPointerException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }
    } 

    public void testGetTrain() {
        RailwayStation station = new RailwayStationImpl(3);
        assertNull(station.getTrain(1));
        assertNull(station.getTrain(2));
        assertNull(station.getTrain(3));
        station.enter(orientExpress);
        assertEquals(orientExpress,station.getTrain(1));
        assertNull(station.getTrain(2));
        assertNull(station.getTrain(3));
        station.enter(brnenskyDrak);
        assertEquals(orientExpress,station.getTrain(1));
        assertEquals(brnenskyDrak,station.getTrain(2));
        assertNull(station.getTrain(3));
        station.leave(orientExpress);
        assertNull(station.getTrain(1));
        assertEquals(brnenskyDrak,station.getTrain(2));
        assertNull(station.getTrain(3));
        station.leave(brnenskyDrak);
        assertNull(station.getTrain(1));
        assertNull(station.getTrain(2));
        assertNull(station.getTrain(3));

        try {
            station.getTrain(-1);
            fail();
        } catch (IndexOutOfBoundsException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }

        try {
            station.getTrain(0);
            fail();
        } catch (IndexOutOfBoundsException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }

        try {
            station.getTrain(4);
            fail();
        } catch (IndexOutOfBoundsException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }
    } 

    public void testEnter() {
        RailwayStation station = new RailwayStationImpl(2);

        try {
            station.enter(null);
            fail();
        } catch (NullPointerException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }

        station.enter(orientExpress);
        assertEquals(orientExpress,station.getTrain(1));

        try {
            station.enter(orientExpress);
            fail("Povedlo se pridat vlak, ktery jiz ve stanici je");
        } catch (IllegalArgumentException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }
        
        station.enter(brnenskyDrak);
        assertEquals(brnenskyDrak,station.getTrain(2));

        try {
            station.enter(orientExpress);
            fail("Povedlo se pridat vlak, ktery jiz ve stanici je");
        } catch (IllegalArgumentException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }

        try {
            station.enter(vysehrad);
            fail("Povedlo se pridat vlak, i kdyz uz v nadrazi neni zadna volna " +
                    "kolej");
        } catch (NoFreeTrackException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }

        try {
            station.enter(null);
            fail();
        } catch (NullPointerException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }

        station.leave(orientExpress);
        assertNull(station.getTrain(1));

        station.enter(vysehrad);
        assertEquals(vysehrad,station.getTrain(1));
    }

    public void testLeave() {
        RailwayStation station = new RailwayStationImpl(2);

        try {
            station.leave(null);
            fail();
        } catch (NullPointerException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }

        try {
            station.leave(orientExpress);
            fail("Povedlo se odstranit vlak, ktery ve stanici neni");
        } catch (IllegalArgumentException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }

        assertNull(station.getTrain(1));
        station.enter(orientExpress);
        assertEquals(orientExpress,station.getTrain(1));
        station.leave(orientExpress);
        assertNull(station.getTrain(1));

        try {
            station.leave(null);
            fail();
        } catch (NullPointerException ex) {
            assertNotNull(ex.getMessage());
            assertTrue(ex.getMessage().length() > 0);
        }

    }

    
    public void testToString() {
        RailwayStation station = new RailwayStationImpl(4);
        station.enter(vysehrad);
        station.enter(osobak);
        station.enter(brnenskyDrak);
        station.enter(orientExpress);
        station.leave(osobak);
        assertEquals("Tracks: 4, Free Tracks: 1 [" +
                "1: EC 134 Vysehrad, " +
                "2: (none), " +
                "3: IC 570 Brnensky drak, " +
                "4: Orient express]",station.toString());
    }
    
}
